
/*
** (c) 2004 Herbert Poetzl
**
** V0.01	ioctls so far
** V0.02	syscall iattr
**
*/

#define VERSION "V0.02"

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <vserver.h>
#include <linux/vserver/inode.h>
 
#ifdef O_LARGEFILE
#define OPEN_FLAGS (O_RDONLY|O_NONBLOCK|O_LARGEFILE)
#else
#define OPEN_FLAGS (O_RDONLY|O_NONBLOCK)
#endif

static	char	err_flag = 0;

static	char	opt_xid = 0;
static	char	opt_flag = 0;
static	char	opt_mask = 0;
static	char	opt_vers = 0;
static	char	opt_ioctl = 0;
static	char	opt_set = 0;

static	char *	cmd_name = NULL;

static	int 	num_xid = 0;
static	int 	num_mask = 0;
static	int 	num_flag = 0;


/* inode ioctls */

#define FIOC_GETXID     _IOR('x', 1, long)
#define FIOC_SETXID     _IOW('x', 2, long)
#define FIOC_SETXIDJ    _IOW('x', 3, long)

#define FIOC_GETXFLG    _IOR('x', 5, long)
#define FIOC_SETXFLG    _IOW('x', 6, long)


int ioctl_getxid (int fd, int *xid)
{
	int r, c;

	r = ioctl (fd, FIOC_GETXID, &c);
	if (xid)
	    *xid = c;
	return r;
}

int ioctl_setxid (int fd, int xid)
{
	int r, c;

	c = (int) xid;
	r = ioctl (fd, FIOC_SETXID, &c);
	return r;
}

int ioctl_getxflg (int fd, int *flag)
{
	int r, c;

	r = ioctl (fd, FIOC_GETXFLG, &c);
	if (flag)
	    *flag = c;
	return r;
}

int ioctl_setxflg (int fd, int flag)
{
	int r, c;

	c = (int) flag;
	r = ioctl (fd, FIOC_SETXFLG, &c);
	return r;
}


int do_work_ioctl(const char *name)
{
	struct stat buf;
	int fd;

	if (stat(name, &buf) != 0)
	    perror("stat:");
	fd = open(name, OPEN_FLAGS);
	if (fd<0)
	    perror("open:");

	if (opt_xid)
	    if (ioctl_setxid(fd, num_xid))
	        perror("setxid:");
	if (opt_flag)
	    if (ioctl_setxflg(fd, num_flag))
	        perror("setxflg:");
	if (ioctl_getxid(fd, &num_xid))
	    perror("getxid:");
	if (ioctl_getxflg(fd, &num_flag))
	    perror("getxflg:");
	printf("xid=%5d, flags=0x%08x, %s\n",
	    num_xid, num_flag, name);
	close (fd);
	return 0;
}


int get_iattr (const char *path, 
	unsigned int *xid, unsigned int *flags, unsigned int *mask)
{
	struct vcmd_ctx_iattr_v1 data;
	int r;

	data.name = path;
	r = vserver(VCMD_get_iattr, 0, &data);
	if (r)
	    return r;
	if (xid)
	    *xid = data.xid;
	if (flags)
	    *flags = data.flags;
	if (mask)
	    *mask = data.mask;
	return 0;
}

int set_iattr (const char *path, 
	unsigned int xid, unsigned int flags, unsigned int mask)
{
	struct vcmd_ctx_iattr_v1 data;
	int r;

	data.name = path;
	data.xid = xid;
	data.flags = flags;
	data.mask = mask;
	r = vserver(VCMD_set_iattr, 0, &data);
	return r;
}


int do_work(const char *name)
{
	unsigned int xid = 0, flags = 0, mask = 0;

	if (opt_xid) {
	    xid = num_xid;
	    mask |= IATTR_XID;
	}
	if (opt_flag) {
	    flags = num_flag;
	    mask |= IATTR_FLAGS;
	}
	if (opt_mask)
	    mask = num_mask;
	if (opt_set)
	    if (set_iattr(name, xid, flags, mask))
		perror("set_iattr:");
	if (get_iattr(name, &xid, &flags, &mask))
	    perror("get_iattr:");

	printf("xid=%5d, flags=0x%08x, mask=0x%08x, %s\n",
	    xid, flags, mask, name);
	return 0;
}


#define OPTIONS "+hif:x:m:V"

int	main(int argc, char *argv[])
{
        extern int optind;
        extern char *optarg;
        char c, errflg = 0;
	int r;


        cmd_name = argv[0];
        while ((c = getopt(argc, argv, OPTIONS)) != EOF) {
            switch (c) {
            case 'h':
                fprintf(stderr,
                    "This is %s " VERSION "\n"
                    "options are:\n"
                    "-h        print this help message\n"
                    "-i        use legacy ioctls\n"
                    "-f <num>  flag value in decimal\n"
                    "-m <num>  option mask\n"
                    "-x <num>  context id\n"
                    "-V        verify interface version\n"
                    "--        end of options\n"
                    ,cmd_name);
                exit(0);
                break;
            case 'i':
                opt_ioctl = 1;
                break;
            case 'f':
                num_flag = atol(optarg);
                opt_flag = 1;
                opt_set = 1;
                break;
            case 'm':
                num_mask = atol(optarg);
                opt_mask = 1;
                opt_set = 1;
                break;
            case 'x':
                num_xid = atol(optarg);
                opt_xid = 1;
                opt_set = 1;
                break;
            case 'V':
                opt_vers = 1;
                break;
            case '?':
            default:
                errflg++;
                break;
            }
        }
        if (errflg) {
            fprintf(stderr, 
                "Usage: %s -[" OPTIONS "]\n"
                "%s -h for help.\n",
                cmd_name, cmd_name);
            exit(2);
        }

	if (opt_vers) {
	    r = vserver(VCMD_get_version, 0, NULL);
	    if (r<0)
	    	perror("vc_get_version");
    	    else 
	    	printf("version: %04x:%04x\n", 
		    (r>>16)&0xFFFF, r&0xFFFF);
	}

        for (; optind < argc; optind++) {
	    const char *name;

            name = argv[optind];

	    if (opt_ioctl)
		do_work_ioctl(name);
	    else
		do_work(name);

	}
	    	
	
	exit((err_flag)?1:0);
}

